<?php

declare(strict_types=1);

namespace DigiComp\PhpCodesniffer\DigiComp\Sniffs\Comments;

use PHP_CodeSniffer\Files\File;
use PHP_CodeSniffer\Sniffs\Sniff;

class DisallowHashCommentsSniff implements Sniff
{
    /**
     * @inheritDoc
     */
    public function register()
    {
        return [\T_COMMENT];
    }

    /**
     * @inheritDoc
     */
    public function process(File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();
        if ($tokens[$stackPtr]['content'][0] === '#') {
            $error = 'Hash comments are prohibited; found %s';
            $fix = $phpcsFile->addFixableError($error, $stackPtr, 'Found', [$tokens[$stackPtr]['content']]);

            if ($fix) {
                $phpcsFile->fixer->beginChangeset();
                $phpcsFile->fixer->replaceToken($stackPtr, '//' . \substr($tokens[$stackPtr]['content'], 1));
                $phpcsFile->fixer->endChangeset();
            }
        }
    }
}
